Package: bootGOF
Title: Bootstrap Based Goodness-of-Fit Tests
Version: 0.1.1
Authors@R: 
    c(person(given = "Marsel",
             family = "Scheer",
             role = c("aut", "cre"),
             email = "scheer@freescience.de"),
      person(given = "Gerhard",
             family = "Dikta",
             role = c("aut"),
             email = "dikta@fh-aachen.de"))
Description: Bootstrap based goodness-of-fit tests. It allows
    to perform rigorous statistical tests to check if a chosen
    model family is correct based on the marked empirical
    process. The implemented algorithms are described in
    (Dikta and Scheer (2021) <doi:10.1007/978-3-030-73480-0>)
    and can be applied
    to generalized linear models without any further implementation
    effort. As far as certain linearity conditions are fulfilled
    the resampling scheme are also applicable beyond generalized
    linear models. This is reflected in the software architecture
    which allows to reuse the resampling scheme by implementing
    only certain interfaces for models that are not supported
    natively by the package.
Imports: checkmate (>= 2.0.0), R6 (>= 2.4.1)
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.2
URL: https://github.com/MarselScheer/bootGOF
BugReports: https://github.com/MarselScheer/bootGOF/issues
Suggests: covr, roxygen2, pkgdown, tinytest, mockery, knitr, rmarkdown,
        minpack.lm, MASS
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-08-28 15:59:16 UTC; rstudio
Author: Marsel Scheer [aut, cre],
  Gerhard Dikta [aut]
Maintainer: Marsel Scheer <scheer@freescience.de>
Repository: CRAN
Date/Publication: 2025-08-28 16:30:02 UTC
Built: R 4.6.0; ; 2026-04-25 17:10:20 UTC; unix
