# Generated by roxygen2: do not edit by hand

export(GOF_glm_info_extractor)
export(GOF_glm_sim_param)
export(GOF_glm_trainer)
export(GOF_lm_info_extractor)
export(GOF_lm_sim_param)
export(GOF_lm_trainer)
export(GOF_model)
export(GOF_model_info_extractor)
export(GOF_model_resample)
export(GOF_model_simulator)
export(GOF_model_test)
export(GOF_model_trainer)
export(GOF_sim_wild_rademacher)
export(Rn1_CvM)
export(Rn1_KS)
export(Rn1_statistic)
importFrom(R6,R6Class)
