## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(echo = FALSE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ---- eval = FALSE, echo = FALSE----------------------------------------------
#  suppressMessages(library(plantuml))
#  uml_str <- paste(readLines("design.plantuml"), collapse = "\n")
#  uml <- plantuml::plantuml(x = uml_str)
#  plot(uml, file = "./vignettes/class_diagram.svg")

